library(Kendall)

Kendall.tau<-function(x){Kendall(1:length(x), x)$tau[1]}

acf.lag1<-function(timeseries){x<-acf(timeseries, plot=FALSE, na.action=na.pass)[[1]][2,1,1]}

DecreasingContactSDE<-function(startk=0.5, deltak=0.0001, mu=0.01, delta=0.1, p=0.5,  q=0.8, Nm=10000, Nh=1000, sigma = 0.05, N, T, Hinit, Vinit){
  #Returns matrix containing infected human and infected mosquito time series. 
  #Biting rate decreases linearly to value kstar (value of biting rate at equilibrium when R0 =1)
  #Demographic stochasticity only, need to add environmental stochasticity
  
  #Args:
  #startk = starting value for the biting rate
  #deltak = change in biting rate
  #q = prob human infects mosquito
  #p=prob mosquito infects human
  #mu = recovery rate per human
  #delta = mortality rate of mosquitoes
  #Nm = number of mosquitoes
  #Nh = number of humans (Nm/Nh = vector to host ratio)
  #sigma = strength of environmental noise in the biting rate
  #N = number of time steps in the simulation
  #T = number of days in the simulation (Dt = T/N)
  #Hinit = human initial condition (equilibrium at startk value)
  #Vinit = vector initial condition
  #Returns:
  #1st col = time, 2nd col = human (H) , 3rd col= vector (V)
  ###########################################
  
  H<-numeric(N+1) #arrays for H and V
  V<-numeric(N+1)
  H[1]<-Hinit
  V[1]<-Vinit
  Dt<-T/N #time increment
  Z1<-rnorm(N, 0, 1)
  Z2<-rnorm(N, 0, 1) #need two standard Wiener processes for demo stochasticity
  Z3<-rnorm(N, 0, 1) #need Wiener process for enviro stochasticity
  ###########Calculate value of k at equilibrium when R0 = 1#######
  
  CalcKstar <- function(params){
    with(                                   #we can simplify code using "with"
      as.list(params),                   #this argument to "with" lets us use the variable names
{
  kstar  <- sqrt((Nh*delta*mu)/(p*q*Nm))
}
    )
  }
  
  params<-c(startk, deltak, mu, delta, p,  q, Nm, Nh)
  
  kstar <- CalcKstar(params)
  
  #####Biting rate function###########

  ##Assume biting rate is a linear function of time
  bitingrate<-function(t){
    
    val<-startk - deltak*t #linear in time
    
    #fn decreases linearly to kstar, then remains constant at this value

    ifelse(val<=kstar, return(kstar), return(val)) }
  
  time<-seq(0, T, Dt)
  k<-sapply(time, bitingrate) #vector of biting rates
  
  
  #####Euler method for system of SDES########
  
  for(i in 1:(N)){
    
    if(H[i]<=0 & V[i]>0){
      #Assume H[i] = 0 for all i
      #calculate covariance matrix
      V11<- k[i]*V[i]*p*(Nh-0)/Nh 
      V12<-0
      V21<-V12
      V22<- delta*V[i]
      #Now find the square root of the covariance matrix, if non-zero along non-diagonal entries use the following algorithm
      #w<-sqrt(V11*V22-V12*V21)
      #d<-sqrt(V11+V22+2*w)
      #B11<-(V11+w)/d
      #B12<-V12/d
      #B21<-B12
      #B22<-(V22+w)/d
      B11<-sqrt(V11)
      B12<-0
      B21<-0
      B22<-sqrt(V22)
      
      #enviro stochasticity in the biting rate
      
      B13 <- sigma*V[i]*p*(Nh)/Nh
      B23 <- 0
      
      #update H and V using Euler method
      meanH <-  k[i]*V[i]*p*(Nh)/Nh
      meanV <-   - delta*V[i] 
      
      tmp<-meanH*Dt+B11*sqrt(Dt)*Z1[i]+B12*sqrt(Dt)*Z2[i]+B13*sqrt(Dt)*Z3[i]
      tmp<-ifelse(tmp<=0, 0, tmp)
                                                
      H[i+1]<-tmp
      V[i+1]<-V[i]+meanV*Dt+B21*sqrt(Dt)*Z1[i]+B22*sqrt(Dt)*Z2[i]+B23*sqrt(Dt)*Z3[i]
      #break
    }
    else if(V[i]<=0 & H[i]<=0){
      #dfe
      H[i+1]<-0
      V[i+1]<-0
    }
    else if(H[i]>Nh){
      #reset number of hosts to Nh - assume that H \leq Nh is a reflecting boundary
      H[i+1]<-Nh
      V[i+1]<-V[i]
    }
    else if(V[i]<=0 & H[i]>0){
      #Assume V[i] = 0 for all i
      #calculate covariance matrix
      V11<-  mu*H[i]
      V12<-0
      V21<-V12
      V22<- k[i]*H[i]*q*(Nm)/Nh 
      #Now find the square root of the covariance matrix 
      #w<-sqrt(V11*V22-V12*V21)
      #d<-sqrt(V11+V22+2*w)
      #B11<-(V11+w)/d
      #B12<-V12/d
      #B21<-B12
      #B22<-(V22+w)/d
      B11<-sqrt(V11)
      B12<-0
      B21<-0
      B22<-sqrt(V22)
      
      #enviro stochasticity in the biting rate
      
      B13 <- 0
     # B23 <- sigma*H[i]*q*(Nm)/Nh
      #update H and V using Euler method
            
      meanH <-   - mu*H[i]
      meanV <-  k[i]*H[i]*q*(Nm)/Nh 
     
     tmp<-meanV*Dt+B21*sqrt(Dt)*Z1[i]+B22*sqrt(Dt)*Z2[i]+B23*sqrt(Dt)*Z3[i]
     tmp<-ifelse(tmp<=0, 0, tmp)
                                                
      H[i+1]<-H[i]+meanH*Dt+B11*sqrt(Dt)*Z1[i]+B12*sqrt(Dt)*Z2[i]+B13*sqrt(Dt)*Z3[i]
      V[i+1]<-tmp #all mosquitoes are dead
        #V[i]+meanV*Dt+B21*sqrt(Dt)*Z1[i]+B22*sqrt(Dt)*Z2[i]+B23*sqrt(Dt)*Z3[i]
    
      
    } else if(H[i]>0 & V[i]>0){
      #calculate covariance matrix 
      V11<- k[i]*V[i]*p*(Nh-H[i])/Nh + mu*H[i]
      V12<-0
      V21<-V12
      V22<- k[i]*H[i]*q*(Nm-V[i])/Nh + delta*V[i]
      #Now find the square root of the covariance matrix 
      #w<-sqrt(V11*V22-V12*V21)
      #d<-sqrt(V11+V22+2*w)
      #B11<-(V11+w)/d
      #B12<-V12/d
      #B21<-B12
      #B22<-(V22+w)/d
      B11<-sqrt(V11)
      B12<-0
      B21<-0
      B22<-sqrt(V22)
      
      #enviro stochasticity in the biting rate
      
      B13 <- sigma*V[i]*p*(Nh-H[i])/Nh
      B23 <- sigma*H[i]*q*(Nm-V[i])/Nh
      
      #update H and V using Euler method
      
      meanH <-  k[i]*V[i]*p*(Nh-H[i])/Nh - mu*H[i]
      meanV <-  k[i]*H[i]*q*(Nm-V[i])/Nh - delta*V[i] 
      
      tmp1<-H[i]+meanH*Dt+B11*sqrt(Dt)*Z1[i]+B12*sqrt(Dt)*Z2[i]+B13*sqrt(Dt)*Z3[i]
      tmp1<-ifelse(tmp1<=0, 0, tmp1)
      tmp1<-ifelse(is.na(tmp1), 0, tmp1)
      tmp2<-V[i]+meanV*Dt+B21*sqrt(Dt)*Z1[i]+B22*sqrt(Dt)*Z2[i]+B23*sqrt(Dt)*Z3[i]
      tmp2<-ifelse(tmp2<=0, 0, tmp2)
      tmp2<-ifelse(is.na(tmp2), 0, tmp2)
      H[i+1]<-tmp1
      V[i+1]<-tmp2
      
     
      
    }
  }
  mat<-matrix(c(time,H,V), ncol=3)
  #return(mat)
  mat.df<-data.frame(time=mat[,1], H=mat[,2],V=mat[,3])
  return(mat.df) #returns as data frame
}

DecreasingRecoverySDE<-function(startmu=0.5, deltamu=0.0001, k=0.5, delta=0.1, p=0.5,  q=0.8, Nm=10000, Nh=1000, sigma = 0.05, N, T, Hinit, Vinit){
  #Returns matrix containing infected human and infected mosquito time series. 
  #Recovery rate decreases linearly to value mustar (value of recovery rate at equilibrium when R0 =1)
  #Demographic stochasticity and environmental stochasticity
  
  #Args:
  #startmu = starting value for the recovery rate
  #deltamu = change in recovery rate
  #k = per capita biting rate
  #q = prob human infects mosquito
  #p=prob mosquito infects human
  #delta = mortality rate of mosquitoes
  #Nm = number of mosquitoes
  #Nh = number of humans (Nm/Nh = vector to host ratio)
  #sigma = strength of environmental noise in the biting rate
  #N = number of time steps in the simulation
  #T = number of days in the simulation (Dt = T/N)
  #Hinit = human initial condition (equilibrium at startk value)
  #Vinit = vector initial condition
  #Returns:
  #1st col = time, 2nd col = human (H) , 3rd col= vector (V)
  ###########################################
  
  H<-numeric(N+1) #arrays for H and V
  V<-numeric(N+1)
  H[1]<-Hinit
  V[1]<-Vinit
  Dt<-T/N #time increment
  Z1<-rnorm(N, 0, 1)
  Z2<-rnorm(N, 0, 1) #need two standard Wiener processes for demo stochasticity
  Z3<-rnorm(N, 0, 1) #need Wiener process for enviro stochasticity
  
  ###########Calculate value of mu at equilibrium when R0 = 1#######
  
  CalcMustar <- function(params){
    with(                                   #we can simplify code using "with"
      as.list(params),                   #this argument to "with" lets us use the variable names
{
  mustar  <- (k*k*p*q*Nm)/(Nh*delta)
}
    )
  }

params<-c(startmu, deltamu, k, delta, p,  q, Nm, Nh)

mustar <- CalcMustar(params)

#####Recovery rate function###########

##Assume recovery rate is a linear function of time
recoveryrate<-function(t){
  
  val<-startmu + deltamu*t #linear in time
  
  #fn decreases linearly to kstar, then remains constant at this value
  
  ifelse(val>=mustar, return(mustar), return(val)) }

time<-seq(0, T, Dt)
mu<-sapply(time, recoveryrate) #vector of biting rates


#####Euler method for system of SDES########

for(i in 1:N){
  
  
  
  if(H[i]<=0 & V[i]>0){
    #Assume H[i] = 0 for all i
    #calculate covariance matrix
    V11<- k*V[i]*p*(Nh-0)/Nh 
    V12<-0
    V21<-V12
    V22<- delta*V[i]
    #Now find the square root of the covariance matrix, if non-zero along non-diagonal entries use the following algorithm
    #w<-sqrt(V11*V22-V12*V21)
    #d<-sqrt(V11+V22+2*w)
    #B11<-(V11+w)/d
    #B12<-V12/d
    #B21<-B12
    #B22<-(V22+w)/d
    B11<-sqrt(V11)
    B12<-0
    B21<-0
    B22<-sqrt(V22)
    
    #enviro stochasticity in the recovery rate
    
    B13 <- 0
    B23 <- 0
    
    #update H and V using Euler method
    meanH <-  k*V[i]*p*(Nh)/Nh
    meanV <-   - delta*V[i] 
    
    tmp<-meanH*Dt+B11*sqrt(Dt)*Z1[i]+B12*sqrt(Dt)*Z2[i]+B13*sqrt(Dt)*Z3[i]
    tmp<-ifelse(tmp<=0, 0, tmp)
    
    
    H[i+1]<-tmp #correct??
    V[i+1]<-V[i]+meanV*Dt+B21*sqrt(Dt)*Z1[i]+B22*sqrt(Dt)*Z2[i]+B23*sqrt(Dt)*Z3[i]
    #break
  }
  else if(V[i]<=0 & H[i]>0){
    #Assume V[i] = 0 for all i
    #calculate covariance matrix
    V11<-  mu[i]*H[i]
    V12<-0
    V21<-V12
    V22<- k*H[i]*q*(Nm)/Nh 
    #Now find the square root of the covariance matrix 
    #w<-sqrt(V11*V22-V12*V21)
    #d<-sqrt(V11+V22+2*w)
    #B11<-(V11+w)/d
    #B12<-V12/d
    #B21<-B12
    #B22<-(V22+w)/d
    B11<-sqrt(V11)
    B12<-0
    B21<-0
    B22<-sqrt(V22)
    
    #enviro stochasticity in the recovery rate
    
    B13 <- sigma*H[i]
    B23 <- 0
    #update H and V using Euler method
    
    meanH <-   - mu[i]*H[i]
    meanV <-  k*H[i]*q*(Nm)/Nh 
   tmp<-meanV*Dt+B21*sqrt(Dt)*Z1[i]+B22*sqrt(Dt)*Z2[i]+B23*sqrt(Dt)*Z3[i]
   tmp<-ifelse(tmp<=0, 0, tmp)
    
    H[i+1]<-H[i]+meanH*Dt+B11*sqrt(Dt)*Z1[i]+B12*sqrt(Dt)*Z2[i]+B13*sqrt(Dt)*Z3[i]
    V[i+1]<-tmp #V[i]+meanV*Dt+B21*sqrt(Dt)*Z1[i]+B22*sqrt(Dt)*Z2[i]+B23*sqrt(Dt)*Z3[i]
    
    
  } else if(V[i]<=0 & H[i]<=0){
    #dfe
    H[i+1]<-0
    V[i+1]<-0
    }
  else if(H[i]>Nh){
    #reset number of hosts to Nh - assume that H \leq Nh is a reflecting boundary
    H[i+1]<-Nh
    V[i+1]<-V[i]
  }
  else if(H[i]>0 & V[i]>0){
    #calculate covariance matrix 
    V11<- k*V[i]*p*(Nh-H[i])/Nh + mu[i]*H[i]
    V12<-0
    V21<-V12
    V22<- k*H[i]*q*(Nm-V[i])/Nh + delta*V[i]
    #Now find the square root of the covariance matrix 
    #w<-sqrt(V11*V22-V12*V21)
    #d<-sqrt(V11+V22+2*w)
    #B11<-(V11+w)/d
    #B12<-V12/d
    #B21<-B12
    #B22<-(V22+w)/d
    B11<-sqrt(V11)
    B12<-0
    B21<-0
    B22<-sqrt(V22)
    
    #enviro stochasticity in the recovery rate
    
    B13 <- sigma*H[i]
    B23 <- 0
    
    #update H and V using Euler method
    
    meanH <-  k*V[i]*p*(Nh-H[i])/Nh - mu[i]*H[i]
    meanV <-  k*H[i]*q*(Nm-V[i])/Nh - delta*V[i] 
    
    tmp1<-H[i]+meanH*Dt+B11*sqrt(Dt)*Z1[i]+B12*sqrt(Dt)*Z2[i]+B13*sqrt(Dt)*Z3[i]
    tmp1<-ifelse(tmp1<=0, 0, tmp1)
    tmp1<-ifelse(is.na(tmp1), 0, tmp1)
    tmp2<-V[i]+meanV*Dt+B21*sqrt(Dt)*Z1[i]+B22*sqrt(Dt)*Z2[i]+B23*sqrt(Dt)*Z3[i]
    tmp2<-ifelse(tmp2<=0, 0, tmp2)
    tmp2<-ifelse(is.na(tmp2), 0, tmp2)
    H[i+1]<-tmp1
    V[i+1]<-tmp2
    
  
}
}
mat<-matrix(c(time,H,V), ncol=3)
#return(mat)
mat.df<-data.frame(time=mat[,1], H=mat[,2],V=mat[,3])
return(mat.df) #returns as data frame
}

DecreasingMortalityRateSDE<-function(startdelta=0.1, deltadelta=0.0001, k=0.5, mu=0.05, p=0.5,  q=0.8, Nm=10000, Nh=1000, sigma = 0.05, N, T, Hinit, Vinit){
  #Returns matrix containing infected human and infected mosquito time series. 
  #Mosquito mortality rate decreases linearly to value deltastar (value of mosquito mortality rate at equilibrium when R0 =1)
  #Demographic stochasticity and environmental stochasticity
  
  #Args:
  #startdelta = starting value for the mosquito mortality rate
  #deltadelta = change in mosquito mortality rate
  #k = per capita biting rate
  #q = prob human infects mosquito
  #p=prob mosquito infects human
  #mu = recovery rate per human
  #Nm = number of mosquitoes
  #Nh = number of humans (Nm/Nh = vector to host ratio)
  #sigma = strength of environmental noise in the biting rate
  #N = number of time steps in the simulation
  #T = number of days in the simulation (Dt = T/N)
  #Hinit = human initial condition (equilibrium at startk value)
  #Vinit = vector initial condition
  #Returns:
  #1st col = time, 2nd col = human (H) , 3rd col= vector (V)
  ###########################################
  
  H<-numeric(N+1) #arrays for H and V
  V<-numeric(N+1)
  H[1]<-Hinit
  V[1]<-Vinit
  Dt<-T/N #time increment
  Z1<-rnorm(N, 0, 1)
  Z2<-rnorm(N, 0, 1) #need two standard Wiener processes for demo stochasticity
  Z3<-rnorm(N, 0, 1) #need Wiener process for enviro stochasticity
  
  ###########Calculate value of mu at equilibrium when R0 = 1#######
  
  CalcDeltastar <- function(params){
    with(                                   #we can simplify code using "with"
      as.list(params),                   #this argument to "with" lets us use the variable names
{
  deltastar  <- (k*k*p*q*Nm)/(Nh*mu)
}
)
  }

params<-c(startdelta, deltadelta, k, mu, p,  q, Nm, Nh)

deltastar <- CalcDeltastar(params)

#####Mortality rate function###########

##Assume recovery rate is a linear function of time
mortalityrate<-function(t){
  
  val<-startdelta + deltadelta*t #linear in time
  
  #fn decreases linearly to kstar, then remains constant at this value
  
  ifelse(val>=deltastar, return(deltastar), return(val)) }

time<-seq(0, T, Dt)
delta<-sapply(time, mortalityrate) #vector of biting rates


#####Euler method for system of SDES########

for(i in 1:N){
  
  if(H[i]<=0 & V[i]>0){
    #Assume H[i] = 0 for all i
    #calculate covariance matrix
    V11<- k*V[i]*p*(Nh-0)/Nh 
    V12<-0
    V21<-V12
    V22<- delta[i]*V[i]
    #Now find the square root of the covariance matrix, if non-zero along non-diagonal entries use the following algorithm
    #w<-sqrt(V11*V22-V12*V21)
    #d<-sqrt(V11+V22+2*w)
    #B11<-(V11+w)/d
    #B12<-V12/d
    #B21<-B12
    #B22<-(V22+w)/d
    B11<-sqrt(V11)
    B12<-0
    B21<-0
    B22<-sqrt(V22)
    
    #enviro stochasticity in the recovery rate
    
    B13 <- 0
    B23 <- sigma*V[i]
    
    #update H and V using Euler method
    meanH <-  k*V[i]*p*(Nh)/Nh
    meanV <-   - delta[i]*V[i] 
    
    tmp<-meanH21*Dt+B11*sqrt(Dt)*Z1[i]+B12*sqrt(Dt)*Z2[i]+B13*sqrt(Dt)*Z3[i]
    tmp<-ifelse(tmp<=0, 0, tmp)
    
    H[i+1]<-tmp
    V[i+1]<-V[i]+meanV*Dt+B21*sqrt(Dt)*Z1[i]+B22*sqrt(Dt)*Z2[i]+B23*sqrt(Dt)*Z3[i]
    #break
  }
  else if(V[i]<=0 & H[i]>0){
    #Assume V[i] = 0 for all i
    #calculate covariance matrix
    V11<-  mu*H[i]
    V12<-0
    V21<-V12
    V22<- k*H[i]*q*(Nm)/Nh 
    #Now find the square root of the covariance matrix 
    #w<-sqrt(V11*V22-V12*V21)
    #d<-sqrt(V11+V22+2*w)
    #B11<-(V11+w)/d
    #B12<-V12/d
    #B21<-B12
    #B22<-(V22+w)/d
    B11<-sqrt(V11)
    B12<-0
    B21<-0
    B22<-sqrt(V22)
    
    #enviro stochasticity in the recovery rate
    
    B13 <- 0
    B23 <- 0
    #update H and V using Euler method
    
    meanH <-   - mu*H[i]
    meanV <-  k*H[i]*q*(Nm)/Nh 
    
    tmp<-meanV*Dt+B21*sqrt(Dt)*Z1[i]+B22*sqrt(Dt)*Z2[i]+B23*sqrt(Dt)*Z3[i] #susceptible mosquitoes may still be infected by infected hosts, even when V[i] = 0.
    tmp<-ifelse(tmp<=0, 0, tmp)
    
    H[i+1]<-H[i]+meanH*Dt+B11*sqrt(Dt)*Z1[i]+B12*sqrt(Dt)*Z2[i]+B13*sqrt(Dt)*Z3[i]
    V[i+1]<-tmp#V[i]+meanV*Dt+B21*sqrt(Dt)*Z1[i]+B22*sqrt(Dt)*Z2[i]+B23*sqrt(Dt)*Z3[i]
    
    
  } else if(V[i]<=0 & H[i]<=0){
    #dfe
    H[i+1]<-0
    V[i+1]<-0
  }
  else if(H[i]>Nh){
    #reset number of hosts to Nh - assume that H \leq Nh is a reflecting boundary
    H[i+1]<-Nh
    V[i+1]<-V[i]
  }else if(H[i]>0 & V[i]>0){
    #calculate covariance matrix 
    V11<- k*V[i]*p*(Nh-H[i])/Nh + mu*H[i]
    V12<-0
    V21<-V12
    V22<- k*H[i]*q*(Nm-V[i])/Nh + delta[i]*V[i]
    #Now find the square root of the covariance matrix 
    #w<-sqrt(V11*V22-V12*V21)
    #d<-sqrt(V11+V22+2*w)
    #B11<-(V11+w)/d
    #B12<-V12/d
    #B21<-B12
    #B22<-(V22+w)/d
    B11<-sqrt(V11)
    B12<-0
    B21<-0
    B22<-sqrt(V22)
    
    #enviro stochasticity in the recovery rate
    
    B13 <- 0
    B23 <- sigma*V[i]
    
    #update H and V using Euler method
    
    meanH <-  k*V[i]*p*(Nh-H[i])/Nh - mu*H[i]
    meanV <-  k*H[i]*q*(Nm-V[i])/Nh - delta[i]*V[i] 
    
    tmp1<-H[i]+meanH*Dt+B11*sqrt(Dt)*Z1[i]+B12*sqrt(Dt)*Z2[i]+B13*sqrt(Dt)*Z3[i]
    tmp1<-ifelse(tmp1<=0, 0, tmp1)
    tmp1<-ifelse(is.na(tmp1), 0, tmp1)
    tmp2<-V[i]+meanV*Dt+B21*sqrt(Dt)*Z1[i]+B22*sqrt(Dt)*Z2[i]+B23*sqrt(Dt)*Z3[i]
    tmp2<-ifelse(tmp2<=0, 0, tmp2)
    tmp2<-ifelse(is.na(tmp2), 0, tmp2)
    H[i+1]<-tmp1
    V[i+1]<-tmp2
    
    
  }
}
mat<-matrix(c(time,H,V), ncol=3)
#return(mat)
mat.df<-data.frame(time=mat[,1], H=mat[,2],V=mat[,3])
return(mat.df) #returns as data frame
}


DecreasingPopSizeSDE<-function(startNm=10000, deltaNm=0.0001, mu=0.01, delta=0.1, p=0.5,  q=0.8, k=0.5, Nh=1000, sigma = 0.05, N, T, Hinit, Vinit){
  #Returns matrix containing infected human and infected mosquito time series. 
  #Biting rate decreases linearly to value kstar (value of biting rate at equilibrium when R0 =1)
  #Demographic stochasticity only, need to add environmental stochasticity
  #Args:
  #startNm = starting value for mosquito population size
  #deltaNm = change in population size
  #q = prob human infects mosquito
  #p=prob mosquito infects human
  #mu = recovery rate per human
  #delta = mortality rate of mosquitoes
  #k = biting rate
  #Nh = number of humans (Nm/Nh = vector to host ratio)
  #sigma = strength of environmental noise
  #N = number of time steps in the simulation
  #T = number of days in the simulation (Dt = T/N)
  #Hinit = human initial condition (equilibrium at startNm value)
  #Vinit = vector initial condition
  #Returns:
  #1st col = time, 2nd col = human (H) , 3rd col= vector (V)
  
  ###########################################
  
  H<-numeric(N+1) #arrays for H and V
  V<-numeric(N+1)
  H[1]<-Hinit
  V[1]<-Vinit
  Dt<-T/N #time increment
  Z1<-rnorm(N, 0, 1)
  Z2<-rnorm(N, 0, 1) #need two standard Wiener processes for demo stochasticity
  Z3<-rnorm(N, 0, 1) #Wiener process for environmental stochasticity
  ###########Calculate value of k at equilibrium when R0 = 1#######
  
  CalcNmstar <- function(params){
    with(                                   #we can simplify code using "with"
      as.list(params),                   #this argument to "with" lets us use the variable names
{
  Nmstar  <- (Nh*delta*mu)/(p*q*k*k)
}
    )
  }

params<-c(startNm, deltaNm, k, mu, delta, p,  q, Nh)

Nmstar <- CalcNmstar(params)

#####changing pop size function###########

##Assume mosquito population size is a linear function of time
MosquitoPopSize<-function(t){
  
  val<-startNm - deltaNm*t #linear in time
  
  #fn decreases linearly to kstar, then remains constant at this value
  
  ifelse(val<=Nmstar, return(Nmstar), return(val)) }
  
  time<-seq(0, T, Dt)
  Nm<-sapply(time, MosquitoPopSize) #vector of biting rates
  
  
  #####Euler method for system of SDES########
  
  for(i in 1:N){
  
    if(H[i]<=0 & V[i]>0){
      #Assume H[i] = 0 for all i
      #calculate covariance matrix
      V11<- k*V[i]*p*(Nh-0)/Nh 
      V12<-0
      V21<-V12
      V22<- delta*V[i]
      #Now find the square root of the covariance matrix, if non-zero along non-diagonal entries use the following algorithm
      #w<-sqrt(V11*V22-V12*V21)
      #d<-sqrt(V11+V22+2*w)
      #B11<-(V11+w)/d
      #B12<-V12/d
      #B21<-B12
      #B22<-(V22+w)/d
      B11<-sqrt(V11)
      B12<-0
      B21<-0
      B22<-sqrt(V22)
      #environmental stochasticity in the mosquito pop size
      B13 <- 0
      B23 <- 0 
      #update H and V using Euler method
      meanH <-  k*V[i]*p*(Nh)/Nh
      meanV <-   - delta*V[i] 
      
      tmp<-meanH*Dt+B11*sqrt(Dt)*Z1[i]+B12*sqrt(Dt)*Z2[i]+B13*sqrt(Dt)*Z3[i]
      tmp<-ifelse(tmp<=0, 0, tmp)
      
      H[i+1]<-tmp
      V[i+1]<-V[i]+meanV*Dt+B21*sqrt(Dt)*Z1[i]+B22*sqrt(Dt)*Z2[i]
      #break
    }
    else if(V[i]<=0 & H[i]>0){
      #Assume V[i] = 0 for all i
      #calculate covariance matrix
      V11<-  mu*H[i]
      V12<-0
      V21<-V12
      V22<- k*H[i]*q*(Nm[i])/Nh 
      #Now find the square root of the covariance matrix 
      #w<-sqrt(V11*V22-V12*V21)
      #d<-sqrt(V11+V22+2*w)
      #B11<-(V11+w)/d
      #B12<-V12/d
      #B21<-B12
      #B22<-(V22+w)/d
      B11<-sqrt(V11)
      B12<-0
      B21<-0
      B22<-sqrt(V22)
      
      #environmental stochasticity in the mosquito pop size
      B13 <- 0
      B23 <- k*H[i]*q*sigma/Nh 
      #update H and V using Euler method
      
      meanH <-   - mu*H[i]
      meanV <-  k*H[i]*q*(Nm[i])/Nh 
      
      tmp<-meanV*Dt+B21*sqrt(Dt)*Z1[i]+B22*sqrt(Dt)*Z2[i]+B23*sqrt(Dt)*Z3[i]
      tmp<-ifelse(tmp<=0, 0, tmp)
      
      H[i+1]<-H[i]+meanH*Dt+B11*sqrt(Dt)*Z1[i]+B12*sqrt(Dt)*Z2[i]
      V[i+1]<-tmp#V[i]+meanV*Dt+B21*sqrt(Dt)*Z1[i]+B22*sqrt(Dt)*Z2[i]
      
      
    } else if(V[i]<=0 & H[i]<=0){
      #dfe
      H[i+1]<-0
      V[i+1]<-0
    }
    else if(H[i]>Nh){
      #reset number of hosts to Nh - assume that H \leq Nh is a reflecting boundary
      H[i+1]<-Nh
      V[i+1]<-V[i]
    }else if(H[i]>0 & V[i]>0){
      #calculate covariance matrix 
      V11<- k*V[i]*p*(Nh-H[i])/Nh + mu*H[i]
      V12<-0
      V21<-V12
      V22<- k*H[i]*q*(Nm[i]-V[i])/Nh + delta*V[i]
      #Now find the square root of the covariance matrix 
      #w<-sqrt(V11*V22-V12*V21)
      #d<-sqrt(V11+V22+2*w)
      #B11<-(V11+w)/d
      #B12<-V12/d
      #B21<-B12
      #B22<-(V22+w)/d
      B11<-sqrt(V11)
      B12<-0
      B21<-0
      B22<-sqrt(V22)
      
      #environmental stochasticity in the mosquito pop size
      B13 <- 0
      B23 <- k*H[i]*q*sigma/Nh   
      #update H and V using Euler method
      
      meanH <-  k*V[i]*p*(Nh-H[i])/Nh - mu*H[i]
      meanV <-  k*H[i]*q*(Nm[i]-V[i])/Nh - delta*V[i] 
      
      tmp1<-H[i]+meanH*Dt+B11*sqrt(Dt)*Z1[i]+B12*sqrt(Dt)*Z2[i]+B13*sqrt(Dt)*Z3[i]
      tmp1<-ifelse(tmp1<=0, 0, tmp1)
      tmp1<-ifelse(is.na(tmp1), 0, tmp1)
      tmp2<-V[i]+meanV*Dt+B21*sqrt(Dt)*Z1[i]+B22*sqrt(Dt)*Z2[i]+B23*sqrt(Dt)*Z3[i]
      tmp2<-ifelse(tmp2<=0, 0, tmp2)
      tmp2<-ifelse(is.na(tmp2), 0, tmp2)
      H[i+1]<-tmp1
      V[i+1]<-tmp2
      
    
      
    }
  }
  mat<-matrix(c(time,H,V), ncol=3)
  #return(mat)
  mat.df<-data.frame(time=mat[,1], H=mat[,2],V=mat[,3])
  return(mat.df) #returns as data frame
}



EE<- function(params){
  
  #Computes the endemic equilibrium of the Ross MacDonald system without immigration of #human hosts
  #  Args:
  #   params: vector of parameter values:
  #k = biting rate, q = prob human infects mosquito, p=prob mosquito infects human, mu = recovery rate per human,
  #delta = mortality rate of mosquitoes, Nm = number of mosquitoes, Nh = number of humans (Nm/Nh = vector to host ratio)
  # Returns:
  #The endemic equilibrium of the Ross-Macdonald system
  
  with(                                   #we can simplify code using "with"
    as.list(params),                   #this argument to "with" lets us use the variable names
{
  ee<-c((k*k*p*q*Nm-delta*mu*Nh)/(mu*k*q+k*k*p*q*Nm/Nh),(k*k*p*q*Nm-delta*mu*Nh)/(k*k*p*q+delta*k*p)) #endemic equilibrium vector
}
  )
}

CalcKstar <- function(params){
  with(                                   #we can simplify code using "with"
    as.list(params),                   #this argument to "with" lets us use the variable names
{
  kstar  <- sqrt((Nh*delta*mu)/(p*q*Nm))
}
  )
}

CalcMustar <- function(params){
  with(                                   #we can simplify code using "with"
    as.list(params),                   #this argument to "with" lets us use the variable names
{
  mustar  <- (k*k*p*q*Nm)/(Nh*delta)
}
  )
}

CalcDeltastar <- function(params){
  with(                                   #we can simplify code using "with"
    as.list(params),                   #this argument to "with" lets us use the variable names
{
  deltastar  <- (k*k*p*q*Nm)/(Nh*mu)
}
  )
}

CalcNmstar <- function(params){
  with(                                   #we can simplify code using "with"
    as.list(params),                   #this argument to "with" lets us use the variable names
{
  Nmstar  <- (Nh*delta*mu)/(p*q*k*k)
}
  )
}

CalcTstarK <- function(params, timestep){
  with(                                   #we can simplify code using "with"
    as.list(params),                   #this argument to "with" lets us use the variable names
{
  tstar <- (startk - CalcKstar(params))/deltak #number time units 
  # tstar <- tstar*timestep #time in days
}
  )
}

CalcTstarNm <- function(params, timestep){
  with(                                   #we can simplify code using "with"
    as.list(params),                   #this argument to "with" lets us use the variable names
{
  tstar <- (startNm - CalcNmstar(params))/deltaNm #number time units 
  # tstar <- tstar*timestep #time in days
}
  )
}

CalcTstarmu <- function(params, timestep){
  with(                                   #we can simplify code using "with"
    as.list(params),                   #this argument to "with" lets us use the variable names
{
  tstar <- (CalcMustar(params)-startmu)/deltamu #number time units 
  # tstar <- tstar*timestep #time in days
}
  )
}

CalcTstardelta <- function(params, timestep){
  with(                                   #we can simplify code using "with"
    as.list(params),                   #this argument to "with" lets us use the variable names
{
  tstar <- (CalcDeltastar(params)- startdelta )/deltadelta #number time units 
  # tstar <- tstar*timestep #time in days
}
  )
}


RMModelBitingSDE<-function(k=0.5, mu=0.01, delta=0.1, p=0.5,  q=0.8, Nm=10000, Nh=1000, sigma = 0.05, N, T, Hinit, Vinit){
  #Returns matrix containing infected human and infected mosquito time series. 
  #Stochasticity about endemic equilibrium
  #Demographic stochasticity only, need to add environmental stochasticity
  
  #Args:
  #k= biting rate
  #q = prob human infects mosquito
  #p=prob mosquito infects human
  #mu = recovery rate per human
  #delta = mortality rate of mosquitoes
  #Nm = number of mosquitoes
  #Nh = number of humans (Nm/Nh = vector to host ratio)
  #sigma = strength of environmental noise in the biting rate
  #N = number of time steps in the simulation
  #T = number of days in the simulation (Dt = T/N)
  #Hinit = human initial condition (equilibrium at startk value)
  #Vinit = vector initial condition
  #Returns:
  #1st col = time, 2nd col = human (H) , 3rd col= vector (V)
  ###########################################
  
  H<-numeric(N+1) #arrays for H and V
  V<-numeric(N+1)
  H[1]<-Hinit
  V[1]<-Vinit
  Dt<-T/N #time increment
  Z1<-rnorm(N, 0, 1)
  Z2<-rnorm(N, 0, 1) #need two standard Wiener processes for demo stochasticity
  Z3<-rnorm(N, 0, 1) #need Wiener process for enviro stochasticity
  
  
  time<-seq(0, T, Dt)
  

#####Euler method for system of SDES########

for(i in 1:(N)){
  
  if(H[i]<=0 & V[i]>0){
    #Assume H[i] = 0 for all i
    #calculate covariance matrix
    V11<- k*V[i]*p*(Nh-0)/Nh 
    V12<-0
    V21<-V12
    V22<- delta*V[i]
    #Now find the square root of the covariance matrix, if non-zero along non-diagonal entries use the following algorithm
    #w<-sqrt(V11*V22-V12*V21)
    #d<-sqrt(V11+V22+2*w)
    #B11<-(V11+w)/d
    #B12<-V12/d
    #B21<-B12
    #B22<-(V22+w)/d
    B11<-sqrt(V11)
    B12<-0
    B21<-0
    B22<-sqrt(V22)
    
    #enviro stochasticity in the biting rate
    
    B13 <- sigma*V[i]*p*(Nh)/Nh
    B23 <- 0
    
    #update H and V using Euler method
    meanH <-  k*V[i]*p*(Nh)/Nh
    meanV <-   - delta*V[i] 
    
    tmp<-meanH*Dt+B11*sqrt(Dt)*Z1[i]+B12*sqrt(Dt)*Z2[i]+B13*sqrt(Dt)*Z3[i]
    tmp<-ifelse(tmp<=0, 0, tmp)
    
    H[i+1]<-tmp
    V[i+1]<-V[i]+meanV*Dt+B21*sqrt(Dt)*Z1[i]+B22*sqrt(Dt)*Z2[i]+B23*sqrt(Dt)*Z3[i]
    #break
  }
  else if(V[i]<=0 & H[i]<=0){
    #dfe
    H[i+1]<-0
    V[i+1]<-0
  }
  else if(H[i]>Nh){
    #reset number of hosts to Nh - assume that H \leq Nh is a reflecting boundary
    H[i+1]<-Nh
    V[i+1]<-V[i]
  }
  else if(V[i]<=0 & H[i]>0){
    #Assume V[i] = 0 for all i
    #calculate covariance matrix
    V11<-  mu*H[i]
    V12<-0
    V21<-V12
    V22<- k*H[i]*q*(Nm)/Nh 
    #Now find the square root of the covariance matrix 
    #w<-sqrt(V11*V22-V12*V21)
    #d<-sqrt(V11+V22+2*w)
    #B11<-(V11+w)/d
    #B12<-V12/d
    #B21<-B12
    #B22<-(V22+w)/d
    B11<-sqrt(V11)
    B12<-0
    B21<-0
    B22<-sqrt(V22)
    
    #enviro stochasticity in the biting rate
    
    B13 <- 0
    # B23 <- sigma*H[i]*q*(Nm)/Nh
    #update H and V using Euler method
    
    meanH <-   - mu*H[i]
    meanV <-  k*H[i]*q*(Nm)/Nh 
    
    tmp<-meanV*Dt+B21*sqrt(Dt)*Z1[i]+B22*sqrt(Dt)*Z2[i]+B23*sqrt(Dt)*Z3[i]
    tmp<-ifelse(tmp<=0, 0, tmp)
    
    H[i+1]<-H[i]+meanH*Dt+B11*sqrt(Dt)*Z1[i]+B12*sqrt(Dt)*Z2[i]+B13*sqrt(Dt)*Z3[i]
    V[i+1]<-tmp #all mosquitoes are dead
    #V[i]+meanV*Dt+B21*sqrt(Dt)*Z1[i]+B22*sqrt(Dt)*Z2[i]+B23*sqrt(Dt)*Z3[i]
    
    
  } else if(H[i]>0 & V[i]>0){
    #calculate covariance matrix 
    V11<- k*V[i]*p*(Nh-H[i])/Nh + mu*H[i]
    V12<-0
    V21<-V12
    V22<- k*H[i]*q*(Nm-V[i])/Nh + delta*V[i]
    #Now find the square root of the covariance matrix 
    #w<-sqrt(V11*V22-V12*V21)
    #d<-sqrt(V11+V22+2*w)
    #B11<-(V11+w)/d
    #B12<-V12/d
    #B21<-B12
    #B22<-(V22+w)/d
    B11<-sqrt(V11)
    B12<-0
    B21<-0
    B22<-sqrt(V22)
    
    #enviro stochasticity in the biting rate
    
    B13 <- sigma*V[i]*p*(Nh-H[i])/Nh
    B23 <- sigma*H[i]*q*(Nm-V[i])/Nh
    
    #update H and V using Euler method
    
    meanH <-  k*V[i]*p*(Nh-H[i])/Nh - mu*H[i]
    meanV <-  k*H[i]*q*(Nm-V[i])/Nh - delta*V[i] 
    
    tmp1<-H[i]+meanH*Dt+B11*sqrt(Dt)*Z1[i]+B12*sqrt(Dt)*Z2[i]+B13*sqrt(Dt)*Z3[i]
    tmp1<-ifelse(tmp1<=0, 0, tmp1)
    tmp1<-ifelse(is.na(tmp1), 0, tmp1)
    tmp2<-V[i]+meanV*Dt+B21*sqrt(Dt)*Z1[i]+B22*sqrt(Dt)*Z2[i]+B23*sqrt(Dt)*Z3[i]
    tmp2<-ifelse(tmp2<=0, 0, tmp2)
    tmp2<-ifelse(is.na(tmp2), 0, tmp2)
    H[i+1]<-tmp1
    V[i+1]<-tmp2
    
    
    
  }
}
mat<-matrix(c(time,H,V), ncol=3)
#return(mat)
mat.df<-data.frame(time=mat[,1], H=mat[,2],V=mat[,3])
return(mat.df) #returns as data frame
}



RMModelPopSizeSDE<-function(Nm=10000, mu=0.01, delta=0.1, p=0.5,  q=0.8, k=0.5, Nh=1000, sigma = 0.05, N, T, Hinit, Vinit){
  #Returns matrix containing infected human and infected mosquito time series. 
  #Demographic stochasticity only, need to add environmental stochasticity
  #Args:
  #Nm = population size
  #q = prob human infects mosquito
  #p=prob mosquito infects human
  #mu = recovery rate per human
  #delta = mortality rate of mosquitoes
  #k = biting rate
  #Nh = number of humans (Nm/Nh = vector to host ratio)
  #sigma = strength of environmental noise
  #N = number of time steps in the simulation
  #T = number of days in the simulation (Dt = T/N)
  #Hinit = human initial condition (equilibrium at startNm value)
  #Vinit = vector initial condition
  #Returns:
  #1st col = time, 2nd col = human (H) , 3rd col= vector (V)
  
  ###########################################
  
  H<-numeric(N+1) #arrays for H and V
  V<-numeric(N+1)
  H[1]<-Hinit
  V[1]<-Vinit
  Dt<-T/N #time increment
  Z1<-rnorm(N, 0, 1)
  Z2<-rnorm(N, 0, 1) #need two standard Wiener processes for demo stochasticity
  Z3<-rnorm(N, 0, 1) #Wiener process for environmental stochasticity
 

time<-seq(0, T, Dt)


#####Euler method for system of SDES########

for(i in 1:N){
  
  if(H[i]<=0 & V[i]>0){
    #Assume H[i] = 0 for all i
    #calculate covariance matrix
    V11<- k*V[i]*p*(Nh-0)/Nh 
    V12<-0
    V21<-V12
    V22<- delta*V[i]
    #Now find the square root of the covariance matrix, if non-zero along non-diagonal entries use the following algorithm
    #w<-sqrt(V11*V22-V12*V21)
    #d<-sqrt(V11+V22+2*w)
    #B11<-(V11+w)/d
    #B12<-V12/d
    #B21<-B12
    #B22<-(V22+w)/d
    B11<-sqrt(V11)
    B12<-0
    B21<-0
    B22<-sqrt(V22)
    #environmental stochasticity in the mosquito pop size
    B13 <- 0
    B23 <- 0 
    #update H and V using Euler method
    meanH <-  k*V[i]*p*(Nh)/Nh
    meanV <-   - delta*V[i] 
    
    tmp<-meanH*Dt+B11*sqrt(Dt)*Z1[i]+B12*sqrt(Dt)*Z2[i]+B13*sqrt(Dt)*Z3[i]
    tmp<-ifelse(tmp<=0, 0, tmp)
    
    H[i+1]<-tmp
    V[i+1]<-V[i]+meanV*Dt+B21*sqrt(Dt)*Z1[i]+B22*sqrt(Dt)*Z2[i]
    #break
  }
  else if(V[i]<=0 & H[i]>0){
    #Assume V[i] = 0 for all i
    #calculate covariance matrix
    V11<-  mu*H[i]
    V12<-0
    V21<-V12
    V22<- k*H[i]*q*(Nm)/Nh 
    #Now find the square root of the covariance matrix 
    #w<-sqrt(V11*V22-V12*V21)
    #d<-sqrt(V11+V22+2*w)
    #B11<-(V11+w)/d
    #B12<-V12/d
    #B21<-B12
    #B22<-(V22+w)/d
    B11<-sqrt(V11)
    B12<-0
    B21<-0
    B22<-sqrt(V22)
    
    #environmental stochasticity in the mosquito pop size
    B13 <- 0
    B23 <- k*H[i]*q*sigma/Nh 
    #update H and V using Euler method
    
    meanH <-   - mu*H[i]
    meanV <-  k*H[i]*q*(Nm)/Nh 
    
    tmp<-meanV*Dt+B21*sqrt(Dt)*Z1[i]+B22*sqrt(Dt)*Z2[i]+B23*sqrt(Dt)*Z3[i]
    tmp<-ifelse(tmp<=0, 0, tmp)
    
    H[i+1]<-H[i]+meanH*Dt+B11*sqrt(Dt)*Z1[i]+B12*sqrt(Dt)*Z2[i]
    V[i+1]<-tmp#V[i]+meanV*Dt+B21*sqrt(Dt)*Z1[i]+B22*sqrt(Dt)*Z2[i]
    
    
  } else if(V[i]<=0 & H[i]<=0){
    #dfe
    H[i+1]<-0
    V[i+1]<-0
  }
  else if(H[i]>Nh){
    #reset number of hosts to Nh - assume that H \leq Nh is a reflecting boundary
    H[i+1]<-Nh
    V[i+1]<-V[i]
  }else if(H[i]>0 & V[i]>0){
    #calculate covariance matrix 
    V11<- k*V[i]*p*(Nh-H[i])/Nh + mu*H[i]
    V12<-0
    V21<-V12
    V22<- k*H[i]*q*(Nm-V[i])/Nh + delta*V[i]
    #Now find the square root of the covariance matrix 
    #w<-sqrt(V11*V22-V12*V21)
    #d<-sqrt(V11+V22+2*w)
    #B11<-(V11+w)/d
    #B12<-V12/d
    #B21<-B12
    #B22<-(V22+w)/d
    B11<-sqrt(V11)
    B12<-0
    B21<-0
    B22<-sqrt(V22)
    
    #environmental stochasticity in the mosquito pop size
    B13 <- 0
    B23 <- k*H[i]*q*sigma/Nh   
    #update H and V using Euler method
    
    meanH <-  k*V[i]*p*(Nh-H[i])/Nh - mu*H[i]
    meanV <-  k*H[i]*q*(Nm-V[i])/Nh - delta*V[i] 
    
    tmp1<-H[i]+meanH*Dt+B11*sqrt(Dt)*Z1[i]+B12*sqrt(Dt)*Z2[i]+B13*sqrt(Dt)*Z3[i]
    tmp1<-ifelse(tmp1<=0, 0, tmp1)
    tmp1<-ifelse(is.na(tmp1), 0, tmp1)
    tmp2<-V[i]+meanV*Dt+B21*sqrt(Dt)*Z1[i]+B22*sqrt(Dt)*Z2[i]+B23*sqrt(Dt)*Z3[i]
    tmp2<-ifelse(tmp2<=0, 0, tmp2)
    tmp2<-ifelse(is.na(tmp2), 0, tmp2)
    H[i+1]<-tmp1
    V[i+1]<-tmp2
    
    
    
  }
}
mat<-matrix(c(time,H,V), ncol=3)
#return(mat)
mat.df<-data.frame(time=mat[,1], H=mat[,2],V=mat[,3])
return(mat.df) #returns as data frame
}

RMModelRecoverySDE<-function(mu=0.5, k=0.5, delta=0.1, p=0.5,  q=0.8, Nm=10000, Nh=1000, sigma = 0.05, N, T, Hinit, Vinit){
  #Returns matrix containing infected human and infected mosquito time series. 
  #Demographic stochasticity and environmental stochasticity
  
  #Args:
  #mu =  recovery rate
  #k = per capita biting rate
  #q = prob human infects mosquito
  #p=prob mosquito infects human
  #delta = mortality rate of mosquitoes
  #Nm = number of mosquitoes
  #Nh = number of humans (Nm/Nh = vector to host ratio)
  #sigma = strength of environmental noise in the biting rate
  #N = number of time steps in the simulation
  #T = number of days in the simulation (Dt = T/N)
  #Hinit = human initial condition (equilibrium at startk value)
  #Vinit = vector initial condition
  #Returns:
  #1st col = time, 2nd col = human (H) , 3rd col= vector (V)
  ###########################################
  
  H<-numeric(N+1) #arrays for H and V
  V<-numeric(N+1)
  H[1]<-Hinit
  V[1]<-Vinit
  Dt<-T/N #time increment
  Z1<-rnorm(N, 0, 1)
  Z2<-rnorm(N, 0, 1) #need two standard Wiener processes for demo stochasticity
  Z3<-rnorm(N, 0, 1) #need Wiener process for enviro stochasticity
  
 
time<-seq(0, T, Dt)


#####Euler method for system of SDES########

for(i in 1:N){
  
  
  
  if(H[i]<=0 & V[i]>0){
    #Assume H[i] = 0 for all i
    #calculate covariance matrix
    V11<- k*V[i]*p*(Nh-0)/Nh 
    V12<-0
    V21<-V12
    V22<- delta*V[i]
    #Now find the square root of the covariance matrix, if non-zero along non-diagonal entries use the following algorithm
    #w<-sqrt(V11*V22-V12*V21)
    #d<-sqrt(V11+V22+2*w)
    #B11<-(V11+w)/d
    #B12<-V12/d
    #B21<-B12
    #B22<-(V22+w)/d
    B11<-sqrt(V11)
    B12<-0
    B21<-0
    B22<-sqrt(V22)
    
    #enviro stochasticity in the recovery rate
    
    B13 <- 0
    B23 <- 0
    
    #update H and V using Euler method
    meanH <-  k*V[i]*p*(Nh)/Nh
    meanV <-   - delta*V[i] 
    
    tmp<-meanH*Dt+B11*sqrt(Dt)*Z1[i]+B12*sqrt(Dt)*Z2[i]+B13*sqrt(Dt)*Z3[i]
    tmp<-ifelse(tmp<=0, 0, tmp)
    
    
    H[i+1]<-tmp #correct??
    V[i+1]<-V[i]+meanV*Dt+B21*sqrt(Dt)*Z1[i]+B22*sqrt(Dt)*Z2[i]+B23*sqrt(Dt)*Z3[i]
    #break
  }
  else if(V[i]<=0 & H[i]>0){
    #Assume V[i] = 0 for all i
    #calculate covariance matrix
    V11<-  mu[i]*H[i]
    V12<-0
    V21<-V12
    V22<- k*H[i]*q*(Nm)/Nh 
    #Now find the square root of the covariance matrix 
    #w<-sqrt(V11*V22-V12*V21)
    #d<-sqrt(V11+V22+2*w)
    #B11<-(V11+w)/d
    #B12<-V12/d
    #B21<-B12
    #B22<-(V22+w)/d
    B11<-sqrt(V11)
    B12<-0
    B21<-0
    B22<-sqrt(V22)
    
    #enviro stochasticity in the recovery rate
    
    B13 <- sigma*H[i]
    B23 <- 0
    #update H and V using Euler method
    
    meanH <-   - mu*H[i]
    meanV <-  k*H[i]*q*(Nm)/Nh 
    tmp<-meanV*Dt+B21*sqrt(Dt)*Z1[i]+B22*sqrt(Dt)*Z2[i]+B23*sqrt(Dt)*Z3[i]
    tmp<-ifelse(tmp<=0, 0, tmp)
    
    H[i+1]<-H[i]+meanH*Dt+B11*sqrt(Dt)*Z1[i]+B12*sqrt(Dt)*Z2[i]+B13*sqrt(Dt)*Z3[i]
    V[i+1]<-tmp #V[i]+meanV*Dt+B21*sqrt(Dt)*Z1[i]+B22*sqrt(Dt)*Z2[i]+B23*sqrt(Dt)*Z3[i]
    
    
  } else if(V[i]<=0 & H[i]<=0){
    #dfe
    H[i+1]<-0
    V[i+1]<-0
  }
  else if(H[i]>Nh){
    #reset number of hosts to Nh - assume that H \leq Nh is a reflecting boundary
    H[i+1]<-Nh
    V[i+1]<-V[i]
  }
  else if(H[i]>0 & V[i]>0){
    #calculate covariance matrix 
    V11<- k*V[i]*p*(Nh-H[i])/Nh + mu*H[i]
    V12<-0
    V21<-V12
    V22<- k*H[i]*q*(Nm-V[i])/Nh + delta*V[i]
    #Now find the square root of the covariance matrix 
    #w<-sqrt(V11*V22-V12*V21)
    #d<-sqrt(V11+V22+2*w)
    #B11<-(V11+w)/d
    #B12<-V12/d
    #B21<-B12
    #B22<-(V22+w)/d
    B11<-sqrt(V11)
    B12<-0
    B21<-0
    B22<-sqrt(V22)
    
    #enviro stochasticity in the recovery rate
    
    B13 <- sigma*H[i]
    B23 <- 0
    
    #update H and V using Euler method
    
    meanH <-  k*V[i]*p*(Nh-H[i])/Nh - mu*H[i]
    meanV <-  k*H[i]*q*(Nm-V[i])/Nh - delta*V[i] 
    
    tmp1<-H[i]+meanH*Dt+B11*sqrt(Dt)*Z1[i]+B12*sqrt(Dt)*Z2[i]+B13*sqrt(Dt)*Z3[i]
    tmp1<-ifelse(tmp1<=0, 0, tmp1)
    tmp1<-ifelse(is.na(tmp1), 0, tmp1)
    tmp2<-V[i]+meanV*Dt+B21*sqrt(Dt)*Z1[i]+B22*sqrt(Dt)*Z2[i]+B23*sqrt(Dt)*Z3[i]
    tmp2<-ifelse(tmp2<=0, 0, tmp2)
    tmp2<-ifelse(is.na(tmp2), 0, tmp2)
    H[i+1]<-tmp1
    V[i+1]<-tmp2
    
    
  }
}
mat<-matrix(c(time,H,V), ncol=3)
#return(mat)
mat.df<-data.frame(time=mat[,1], H=mat[,2],V=mat[,3])
return(mat.df) #returns as data frame
}

RMModelMortalitySDE<-function(delta=0.1,  k=0.5, mu=0.05, p=0.5,  q=0.8, Nm=10000, Nh=1000, sigma = 0.05, N, T, Hinit, Vinit){
  #Returns matrix containing infected human and infected mosquito time series. 
  #Demographic stochasticity and environmental stochasticity
  
  #Args:
  #delta = mosquito mortality rate
  #k = per capita biting rate
  #q = prob human infects mosquito
  #p=prob mosquito infects human
  #mu = recovery rate per human
  #Nm = number of mosquitoes
  #Nh = number of humans (Nm/Nh = vector to host ratio)
  #sigma = strength of environmental noise in the biting rate
  #N = number of time steps in the simulation
  #T = number of days in the simulation (Dt = T/N)
  #Hinit = human initial condition (equilibrium at startk value)
  #Vinit = vector initial condition
  #Returns:
  #1st col = time, 2nd col = human (H) , 3rd col= vector (V)
  ###########################################
  
  H<-numeric(N+1) #arrays for H and V
  V<-numeric(N+1)
  H[1]<-Hinit
  V[1]<-Vinit
  Dt<-T/N #time increment
  Z1<-rnorm(N, 0, 1)
  Z2<-rnorm(N, 0, 1) #need two standard Wiener processes for demo stochasticity
  Z3<-rnorm(N, 0, 1) #need Wiener process for enviro stochasticity
  
 

time<-seq(0, T, Dt)



#####Euler method for system of SDES########

for(i in 1:N){
  
  if(H[i]<=0 & V[i]>0){
    #Assume H[i] = 0 for all i
    #calculate covariance matrix
    V11<- k*V[i]*p*(Nh-0)/Nh 
    V12<-0
    V21<-V12
    V22<- delta*V[i]
    #Now find the square root of the covariance matrix, if non-zero along non-diagonal entries use the following algorithm
    #w<-sqrt(V11*V22-V12*V21)
    #d<-sqrt(V11+V22+2*w)
    #B11<-(V11+w)/d
    #B12<-V12/d
    #B21<-B12
    #B22<-(V22+w)/d
    B11<-sqrt(V11)
    B12<-0
    B21<-0
    B22<-sqrt(V22)
    
    #enviro stochasticity in the recovery rate
    
    B13 <- 0
    B23 <- sigma*V[i]
    
    #update H and V using Euler method
    meanH <-  k*V[i]*p*(Nh)/Nh
    meanV <-   - delta*V[i] 
    
    tmp<-meanH21*Dt+B11*sqrt(Dt)*Z1[i]+B12*sqrt(Dt)*Z2[i]+B13*sqrt(Dt)*Z3[i]
    tmp<-ifelse(tmp<=0, 0, tmp)
    
    H[i+1]<-tmp
    V[i+1]<-V[i]+meanV*Dt+B21*sqrt(Dt)*Z1[i]+B22*sqrt(Dt)*Z2[i]+B23*sqrt(Dt)*Z3[i]
    #break
  }
  else if(V[i]<=0 & H[i]>0){
    #Assume V[i] = 0 for all i
    #calculate covariance matrix
    V11<-  mu*H[i]
    V12<-0
    V21<-V12
    V22<- k*H[i]*q*(Nm)/Nh 
    #Now find the square root of the covariance matrix 
    #w<-sqrt(V11*V22-V12*V21)
    #d<-sqrt(V11+V22+2*w)
    #B11<-(V11+w)/d
    #B12<-V12/d
    #B21<-B12
    #B22<-(V22+w)/d
    B11<-sqrt(V11)
    B12<-0
    B21<-0
    B22<-sqrt(V22)
    
    #enviro stochasticity in the recovery rate
    
    B13 <- 0
    B23 <- 0
    #update H and V using Euler method
    
    meanH <-   - mu*H[i]
    meanV <-  k*H[i]*q*(Nm)/Nh 
    
    tmp<-meanV*Dt+B21*sqrt(Dt)*Z1[i]+B22*sqrt(Dt)*Z2[i]+B23*sqrt(Dt)*Z3[i] #susceptible mosquitoes may still be infected by infected hosts, even when V[i] = 0.
    tmp<-ifelse(tmp<=0, 0, tmp)
    
    H[i+1]<-H[i]+meanH*Dt+B11*sqrt(Dt)*Z1[i]+B12*sqrt(Dt)*Z2[i]+B13*sqrt(Dt)*Z3[i]
    V[i+1]<-tmp#V[i]+meanV*Dt+B21*sqrt(Dt)*Z1[i]+B22*sqrt(Dt)*Z2[i]+B23*sqrt(Dt)*Z3[i]
    
    
  } else if(V[i]<=0 & H[i]<=0){
    #dfe
    H[i+1]<-0
    V[i+1]<-0
  }
  else if(H[i]>Nh){
    #reset number of hosts to Nh - assume that H \leq Nh is a reflecting boundary
    H[i+1]<-Nh
    V[i+1]<-V[i]
  }else if(H[i]>0 & V[i]>0){
    #calculate covariance matrix 
    V11<- k*V[i]*p*(Nh-H[i])/Nh + mu*H[i]
    V12<-0
    V21<-V12
    V22<- k*H[i]*q*(Nm-V[i])/Nh + delta*V[i]
    #Now find the square root of the covariance matrix 
    #w<-sqrt(V11*V22-V12*V21)
    #d<-sqrt(V11+V22+2*w)
    #B11<-(V11+w)/d
    #B12<-V12/d
    #B21<-B12
    #B22<-(V22+w)/d
    B11<-sqrt(V11)
    B12<-0
    B21<-0
    B22<-sqrt(V22)
    
    #enviro stochasticity in the recovery rate
    
    B13 <- 0
    B23 <- sigma*V[i]
    
    #update H and V using Euler method
    
    meanH <-  k*V[i]*p*(Nh-H[i])/Nh - mu*H[i]
    meanV <-  k*H[i]*q*(Nm-V[i])/Nh - delta*V[i] 
    
    tmp1<-H[i]+meanH*Dt+B11*sqrt(Dt)*Z1[i]+B12*sqrt(Dt)*Z2[i]+B13*sqrt(Dt)*Z3[i]
    tmp1<-ifelse(tmp1<=0, 0, tmp1)
    tmp1<-ifelse(is.na(tmp1), 0, tmp1)
    tmp2<-V[i]+meanV*Dt+B21*sqrt(Dt)*Z1[i]+B22*sqrt(Dt)*Z2[i]+B23*sqrt(Dt)*Z3[i]
    tmp2<-ifelse(tmp2<=0, 0, tmp2)
    tmp2<-ifelse(is.na(tmp2), 0, tmp2)
    H[i+1]<-tmp1
    V[i+1]<-tmp2
    
    
  }
}
mat<-matrix(c(time,H,V), ncol=3)
#return(mat)
mat.df<-data.frame(time=mat[,1], H=mat[,2],V=mat[,3])
return(mat.df) #returns as data frame
}

select.periodic <- function(sim,period=10,start=1){
  #changes a time series to output per day 
  #Args:
  #sim <- single time series
  #period <- number of time points in a day
  #start <- starting point of time series
  #Returns:
  # the per day time series
  index <- seq(from=start,to=length(sim),by=period)
  return(sim[index])
}

###########################################################################################
# Analyze1.R
#
# Authors: Arash Fard, Zach McElrath
#
# Purpose: takes the time series array 'X' and calculates the
#  specified statistic 'Function' on sliding windows over X
#  of sizes ranging from 'w1' to 'w2', and then makes an Image
#	plot of the values of the statistic over the various window sizes
#	and returns the normalized matrix of those results.
#	
# Parameters: 	X--the time series array
#			T--an event times array, which will be plotted on the x-axis
#			w2--the maximum window size
#			w1--the minimum window size [default: 3]
#			Lines--a vector of event times at which to display a vertical line
#			Function--the statistic to calculate [default: var (variance)]
#			FunctionString--a character representation of the statistic [default: 'Variance']
# Returns:		StatMatrix-- a matrix of dimension (t x y), where t=length(X) - 1
#					and y=w2-w1+1
# Conditions: 	w2 >= w1 > 1
###############################################################################################

Analyze1 <- function(X, T, w2, w1=3, Lines=NULL, Function=var, FunctionString='Variance'){
  
  # finding the size of array X
  t = length(X)-1
  # The smallest window size is w1; hence, the number of columns in matrix would be w2-w1+1
  y=w2-w1+1
  # The appropriate matrix is allocated and filled by NaN for its all elements
  StatMatrix <- array(NaN, dim=c(t,y)) # t rows, y columns
  NormalMatrix <- array(NaN, dim=c(t,y)) # t rows, y columns
  # The matrix is filled using two nested loops. elements of the input array should not be NaN or inf
  for ( i in 1:y){ # columns
    for (j in (i+w1-1):t){ # looping through the rows
      # Perform the given function (default: standard deviation)
      
      StatMatrix[j,i] = Function(X[(j-i-w1+2):j])
      
    }
    # Numbers corresponding to each window are normalised using their mean
    NormalMatrix[,i] = StatMatrix[,i]/mean(StatMatrix[(i+w1-1):t,i])
  }
  # A graph corresponding to the matrix will be illustrated
  if (w2 > w1){
    # Generate an image() plot
    image(NormalMatrix, axes=FALSE, 
          xlab="Time", ylab="window size", 
          main=FunctionString)
    # Create the x-axis
    maxEventTime <- round(max(T),0)
    axis(1, at=seq(0,1, by=.1), lab=seq(0,maxEventTime,by=.1*maxEventTime), las=1)
    # Create the y-axis
    if( w2-w1 <10) {
      axis(2, at=seq(0,1, by=1/(w2-w1)), lab=seq(w1,w2,by=1), las=1)
    } else {
      increment <- ceiling((w2-w1) / 10)
      axis(2, at=seq(0,1, by=increment/(w2-w1)), lab=seq(w1,w2,by=increment), las=1)
    }
    mtext(paste("window sizes ",w1," to ",w2), line=0.1)
  } else {
    # special case when w2==w1 # replace Normal with Stat - dont want to normalise
    plot(y <-StatMatrix[,1], x=T[1:t], type="l",  col="red",
         xlab="Time", ylab=FunctionString,
         main=FunctionString)
    mtext(paste("fixed window size: ",w1), line=0.1)
  }
  
  
  
  # Drawing vertical lines corresponding to vector of Lines
  if (!is.null(Lines)) {
    if (w2 > w1){
      abline(v=Lines/t)
      axis(1, at=Lines/t, lab=T[Lines], las=1, line=-1, col.axis='red')
    } else {
      abline(v=T[Lines])
      axis(1, at=T[Lines], lab=T[Lines], las=1, line=-1, col.axis='red')
    }
  }
  
  return (StatMatrix)
  
} # end function Analyze1()

process<-function(series, window.size=54, stat.fn="acf.lag1"){
  ###function that calculates a statistic of fluctuations over a moving window
  ##Args: series = vector of residuals
  #window.size = size of window over which to calculate statistic
  #stat.fn = summary statistic (e.g. variance, autocorrelation)
  stat <- rep(NaN, (window.size - 1))
  for(i in window.size:length(series)){
    stat[i] <- get(stat.fn, mode="closure")(series[(i - window.size + 1):i])
  }
  
  return(stat)
}
